@@ -1,6 +1,6 @@
 /*
  * Cppcheck - A tool for static C/C++ code analysis
- * Copyright (C) 2007-2024 Cppcheck team.
+ * Copyright (C) 2007-2025 Cppcheck team.
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -16,11 +16,14 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
+#if defined(__CYGWIN__)
+#define _BSD_SOURCE // required to have getloadavg()
+#endif
+
 #include "processexecutor.h"
 
-#if !defined(WIN32) && !defined(__MINGW32__)
+#ifdef HAS_THREADING_MODEL_FORK
 
-#include "config.h"
 #include "cppcheck.h"
 #include "errorlogger.h"
 #include "errortypes.h"
@@ -28,6 +31,7 @@
 #include "settings.h"
 #include "suppressions.h"
 #include "timer.h"
+#include "utils.h"
 
 #include <algorithm>
 #include <numeric>
@@ -64,7 +68,7 @@ enum class Color : std::uint8_t;
 using std::memset;
 
 
-ProcessExecutor::ProcessExecutor(const std::list<FileWithDetails> &files, const std::list<FileSettings>& fileSettings, const Settings &settings, SuppressionList &suppressions, ErrorLogger &errorLogger, CppCheck::ExecuteCmdFn executeCommand)
+ProcessExecutor::ProcessExecutor(const std::list<FileWithDetails> &files, const std::list<FileSettings>& fileSettings, const Settings &settings, Suppressions &suppressions, ErrorLogger &errorLogger, CppCheck::ExecuteCmdFn executeCommand)
     : Executor(files, fileSettings, settings, suppressions, errorLogger)
     , mExecuteCommand(std::move(executeCommand))
 {
@@ -74,7 +78,7 @@ ProcessExecutor::ProcessExecutor(const std::list<FileWithDetails> &files, const
 namespace {
     class PipeWriter : public ErrorLogger {
     public:
-        enum PipeSignal : std::uint8_t {REPORT_OUT='1',REPORT_ERROR='2', CHILD_END='5'};
+        enum PipeSignal : std::uint8_t {REPORT_OUT='1',REPORT_ERROR='2',REPORT_SUPPR_INLINE='3',REPORT_SUPPR='4',CHILD_END='5',REPORT_METRIC='6'};
 
         explicit PipeWriter(int pipe) : mWpipe(pipe) {}
 
@@ -86,23 +90,49 @@ namespace {
             writeToPipe(REPORT_ERROR, msg.serialize());
         }
 
+        void writeSuppr(const SuppressionList &supprs) const {
+            for (const auto& suppr : supprs.getSuppressions())
+            {
+                if (suppr.isInline)
+                    writeToPipe(REPORT_SUPPR_INLINE, suppressionToString(suppr));
+                else if (suppr.checked)
+                    writeToPipe(REPORT_SUPPR, suppressionToString(suppr));
+            }
+        }
+
+        void reportMetric(const std::string &metric) override {
+            writeToPipe(REPORT_METRIC, metric);
+        }
+
         void writeEnd(const std::string& str) const {
             writeToPipe(CHILD_END, str);
         }
 
     private:
+        static std::string suppressionToString(const SuppressionList::Suppression &suppr)
+        {
+            std::string suppr_str = suppr.toString();
+            suppr_str += ";";
+            suppr_str += suppr.checked ? "1" : "0";
+            suppr_str += ";";
+            suppr_str += suppr.matched ? "1" : "0";
+            suppr_str += ";";
+            suppr_str += suppr.extraComment;
+            return suppr_str;
+        }
+
         // TODO: how to log file name in error?
         void writeToPipeInternal(PipeSignal type, const void* data, std::size_t to_write) const
         {
             const ssize_t bytes_written = write(mWpipe, data, to_write);
             if (bytes_written <= 0) {
                 const int err = errno;
-                std::cerr << "#### ThreadExecutor::writeToPipeInternal() error for type " << type << ": " << std::strerror(err) << std::endl;
+                std::cerr << "#### ThreadExecutor::writeToPipeInternal() error for type " << static_cast<unsigned char>(type) << ": " << std::strerror(err) << std::endl;
                 std::exit(EXIT_FAILURE);
             }
             // TODO: write until everything is written
             if (bytes_written != to_write) {
-                std::cerr << "#### ThreadExecutor::writeToPipeInternal() error for type " << type << ": insufficient data written (expected: " << to_write << " / got: " << bytes_written << ")" << std::endl;
+                std::cerr << "#### ThreadExecutor::writeToPipeInternal() error for type " << static_cast<unsigned char>(type) << ": insufficient data written (expected: " << to_write << " / got: " << bytes_written << ")" << std::endl;
                 std::exit(EXIT_FAILURE);
             }
         }
@@ -120,7 +150,8 @@ namespace {
                 writeToPipeInternal(type, &len, l_size);
             }
 
-            writeToPipeInternal(type, data.c_str(), len);
+            if (len > 0) // TODO: unexpected - write a warning?
+                writeToPipeInternal(type, data.c_str(), len);
         }
 
         const int mWpipe;
@@ -150,7 +181,12 @@ bool ProcessExecutor::handleRead(int rpipe, unsigned int &result, const std::str
         std::exit(EXIT_FAILURE);
     }
 
-    if (type != PipeWriter::REPORT_OUT && type != PipeWriter::REPORT_ERROR && type != PipeWriter::CHILD_END) {
+    if (type != PipeWriter::REPORT_OUT &&
+        type != PipeWriter::REPORT_ERROR &&
+        type != PipeWriter::REPORT_SUPPR_INLINE &&
+        type != PipeWriter::REPORT_SUPPR &&
+        type != PipeWriter::CHILD_END &&
+        type != PipeWriter::REPORT_METRIC) {
         std::cerr << "#### ThreadExecutor::handleRead(" << filename << ") invalid type " << int(type) << std::endl;
         std::exit(EXIT_FAILURE);
     }
@@ -169,18 +205,20 @@ bool ProcessExecutor::handleRead(int rpipe, unsigned int &result, const std::str
     }
 
     std::string buf(len, '\0');
-    char *data_start = &buf[0];
-    bytes_to_read = len;
-    do {
-        bytes_read = read(rpipe, data_start, bytes_to_read);
-        if (bytes_read <= 0) {
-            const int err = errno;
-            std::cerr << "#### ThreadExecutor::handleRead(" << filename << ") error (buf) for type" << int(type) << ": " << std::strerror(err) << std::endl;
-            std::exit(EXIT_FAILURE);
-        }
-        bytes_to_read -= bytes_read;
-        data_start += bytes_read;
-    } while (bytes_to_read != 0);
+    if (len > 0) { // TODO: unexpected - write a warning?
+        char *data_start = &buf[0];
+        bytes_to_read = len;
+        do {
+            bytes_read = read(rpipe, data_start, bytes_to_read);
+            if (bytes_read <= 0) {
+                const int err = errno;
+                std::cerr << "#### ThreadExecutor::handleRead(" << filename << ") error (buf) for type" << int(type) << ": " << std::strerror(err) << std::endl;
+                std::exit(EXIT_FAILURE);
+            }
+            bytes_to_read -= bytes_read;
+            data_start += bytes_read;
+        } while (bytes_to_read != 0);
+    }
 
     bool res = true;
     if (type == PipeWriter::REPORT_OUT) {
@@ -199,9 +237,38 @@ bool ProcessExecutor::handleRead(int rpipe, unsigned int &result, const std::str
 
         if (hasToLog(msg))
             mErrorLogger.reportErr(msg);
+    } else if (type == PipeWriter::REPORT_SUPPR_INLINE || type == PipeWriter::REPORT_SUPPR) {
+        if (!buf.empty()) {
+            // TODO: avoid string splitting
+            auto parts = splitString(buf, ';');
+            if (parts.size() < 4)
+            {
+                // TODO: make this non-fatal
+                std::cerr << "#### ThreadExecutor::handleRead(" << filename << ") adding of inline suppression failed - insufficient data" << std::endl;
+                std::exit(EXIT_FAILURE);
+            }
+            auto suppr = SuppressionList::parseLine(parts[0]);
+            suppr.isInline = (type == PipeWriter::REPORT_SUPPR_INLINE);
+            suppr.checked = parts[1] == "1";
+            suppr.matched = parts[2] == "1";
+            suppr.extraComment = parts[3];
+            for (std::size_t i = 4; i < parts.size(); i++) {
+                suppr.extraComment += ";" + parts[i];
+            }
+            const std::string err = mSuppressions.nomsg.addSuppression(suppr);
+            if (!err.empty()) {
+                // TODO: only update state if it doesn't exist - otherwise propagate error
+                mSuppressions.nomsg.updateSuppressionState(suppr); // TODO: check result
+                // TODO: make this non-fatal
+                //std::cerr << "#### ThreadExecutor::handleRead(" << filename << ") adding of inline suppression failed - " << err << std::endl;
+                //std::exit(EXIT_FAILURE);
+            }
+        }
     } else if (type == PipeWriter::CHILD_END) {
         result += std::stoi(buf);
         res = false;
+    } else if (type == PipeWriter::REPORT_METRIC) {
+        mErrorLogger.reportMetric(buf);
     }
 
     return res;
@@ -209,7 +276,7 @@ bool ProcessExecutor::handleRead(int rpipe, unsigned int &result, const std::str
 
 bool ProcessExecutor::checkLoadAverage(size_t nchildren)
 {
-#if defined(__QNX__) || defined(__HAIKU__)  // getloadavg() is unsupported on Qnx, Haiku.
+#if defined(__QNX__) || defined(__HAIKU__) || defined(_AIX) // getloadavg() is unsupported on Qnx, Haiku, AIX.
     (void)nchildren;
     return true;
 #else
@@ -242,8 +309,8 @@ unsigned int ProcessExecutor::check()
     std::map<pid_t, std::string> childFile;
     std::map<int, std::string> pipeFile;
     std::size_t processedsize = 0;
-    std::list<FileWithDetails>::const_iterator iFile = mFiles.cbegin();
-    std::list<FileSettings>::const_iterator iFileSettings = mFileSettings.cbegin();
+    auto iFile = mFiles.cbegin();
+    auto iFileSettings = mFileSettings.cbegin();
     for (;;) {
         // Start a new child
         const size_t nchildren = childFile.size();
@@ -277,20 +344,18 @@ unsigned int ProcessExecutor::check()
                 close(pipes[0]);
 
                 PipeWriter pipewriter(pipes[1]);
-                CppCheck fileChecker(pipewriter, false, mExecuteCommand);
-                fileChecker.settings() = mSettings;
+                CppCheck fileChecker(mSettings, mSuppressions, pipewriter, false, mExecuteCommand);
                 unsigned int resultOfCheck = 0;
 
                 if (iFileSettings != mFileSettings.end()) {
                     resultOfCheck = fileChecker.check(*iFileSettings);
-                    if (fileChecker.settings().clangTidy)
-                        fileChecker.analyseClangTidy(*iFileSettings);
                 } else {
                     // Read file from a file
                     resultOfCheck = fileChecker.check(*iFile);
-                    // TODO: call analyseClangTidy()?
                 }
 
+                pipewriter.writeSuppr(mSuppressions.nomsg);
+
                 pipewriter.writeEnd(std::to_string(resultOfCheck));
                 std::exit(EXIT_SUCCESS);
             }
@@ -310,7 +375,7 @@ unsigned int ProcessExecutor::check()
         if (!rpipes.empty()) {
             fd_set rfds;
             FD_ZERO(&rfds);
-            for (std::list<int>::const_iterator rp = rpipes.cbegin(); rp != rpipes.cend(); ++rp)
+            for (auto rp = rpipes.cbegin(); rp != rpipes.cend(); ++rp)
                 FD_SET(*rp, &rfds);
             timeval tv; // for every second polling of load average condition
             tv.tv_sec = 1;
@@ -318,11 +383,11 @@ unsigned int ProcessExecutor::check()
             const int r = select(*std::max_element(rpipes.cbegin(), rpipes.cend()) + 1, &rfds, nullptr, nullptr, &tv);
 
             if (r > 0) {
-                std::list<int>::const_iterator rp = rpipes.cbegin();
+                auto rp = rpipes.cbegin();
                 while (rp != rpipes.cend()) {
                     if (FD_ISSET(*rp, &rfds)) {
                         std::string name;
-                        const std::map<int, std::string>::const_iterator p = pipeFile.find(*rp);
+                        const auto p = utils::as_const(pipeFile).find(*rp);
                         if (p != pipeFile.cend()) {
                             name = p->second;
                         }
@@ -358,7 +423,7 @@ unsigned int ProcessExecutor::check()
             const pid_t child = waitpid(0, &stat, WNOHANG);
             if (child > 0) {
                 std::string childname;
-                const std::map<pid_t, std::string>::const_iterator c = childFile.find(child);
+                const auto c = utils::as_const(childFile).find(child);
                 if (c != childFile.cend()) {
                     childname = c->second;
                     childFile.erase(c);
@@ -385,7 +450,7 @@ unsigned int ProcessExecutor::check()
     }
 
     // TODO: wee need to get the timing information from the subprocess
-    if (mSettings.showtime == SHOWTIME_MODES::SHOWTIME_SUMMARY || mSettings.showtime == SHOWTIME_MODES::SHOWTIME_TOP5_SUMMARY)
+    if (mSettings.showtime == ShowTime::SUMMARY || mSettings.showtime == ShowTime::TOP5_SUMMARY)
         CppCheck::printTimerResults(mSettings.showtime);
 
     return result;
@@ -396,14 +461,14 @@ void ProcessExecutor::reportInternalChildErr(const std::string &childname, const
     std::list<ErrorMessage::FileLocation> locations;
     locations.emplace_back(childname, 0, 0);
     const ErrorMessage errmsg(std::move(locations),
-                              emptyString,
+                              "",
                               Severity::error,
                               "Internal error: " + msg,
                               "cppcheckError",
                               Certainty::normal);
 
-    if (!mSuppressions.isSuppressed(errmsg, {}))
+    if (!mSuppressions.nomsg.isSuppressed(errmsg, {}))
         mErrorLogger.reportErr(errmsg);
 }
 
-#endif // !WIN32
+#endif // HAS_THREADING_MODEL_FORK
