@@ -1,4 +1,4 @@
-/*
+/* -*- C++ -*-
  * Cppcheck - A tool for static C/C++ code analysis
  * Copyright (C) 2007-2021 Cppcheck team.
  *
@@ -23,4 +23,9 @@ class TestResultsTree : public QObject {
 
 private slots:
     void test1() const;
+    void multiLineResult() const;
+    void resultsInSameFile() const;
+    void testReportType() const;
+    void testGetGuidelineError() const;
+    void misraCReportShowClassifications() const;
 };
